searchData={"items":[{"type":"behaviour","title":"EctoCursorBasedStream","doc":"Use this module in any module that uses `Ecto.Repo`\nto enrich it with `cursor_based_stream/2` function.\n\nExample:\n\n    defmodule MyRepo do\n      use Ecto.Repo\n      use EctoCursorBasedStream\n    end\n\n    MyUser\n    |> MyRepo.cursor_based_stream(max_rows: 100)\n    |> Stream.each(...)\n    |> Stream.run()","ref":"EctoCursorBasedStream.html"},{"type":"callback","title":"EctoCursorBasedStream.cursor_based_stream/2","doc":"Return a lazy enumerable that emits all entries from the data store\nmatching the given query.\n\nIn contrast to `Ecto.Repo.stream/2`,\nthis will not use database mechanisms (e.g. database transactions) to stream the rows.\n\nIt does so by sorting all the rows by `:cursor_field` and iterating over them in chunks\nof size `:max_rows`.","ref":"EctoCursorBasedStream.html#c:cursor_based_stream/2"},{"type":"callback","title":"Options - EctoCursorBasedStream.cursor_based_stream/2","doc":"* `:cursor_field` - the field or list of fields by which all rows should be iterated.\n\n  **This field must have unique values. (Otherwise, some rows may get skipped.)**\n\n  For performance reasons, we recommend that you have an index on that field. Defaults to `:id`.\n\n* `:after_cursor` - the value of the `:cursor_field` that results start. When `:cursor_field` is a list\n  then`:after_cursor` must be a map where keys are cursor fields (not all fields are required).\n\n  Useful when you want to continue streaming from a certain point.\n  Any rows with value equal or smaller than this value will not be included.\n\n  Defaults to `nil`. (All rows will be included.)\n\n* `:max_rows` - The number of rows to load from the database as we stream.\n\n  Defaults to 500.\n\n* `:order` - Order of results, `:asc` or `:desc`\n\n  Defaults to `:asc`.\n\n* `:parallel` - when `true` fetches next batch of records in parallel to processing the stream.\n\n  Defaults to `false` as this spawns `Task`s and could cause issues e.g. with Ecto Sandbox in tests.\n\n* `:prefix, :timeout, :log, :telemetry_event, :telemetry_options` - options passed directly to `Ecto.Repo.all/2`","ref":"EctoCursorBasedStream.html#c:cursor_based_stream/2-options"},{"type":"callback","title":"Examples - EctoCursorBasedStream.cursor_based_stream/2","doc":"MyUser\n    |> MyRepo.cursor_based_stream(max_rows: 1000)\n    |> Stream.each(...)\n    |> Stream.run()\n\n    # change order, run in parallel\n    MyUser\n    |> MyRepo.cursor_based_stream(order: :desc, parallel: true)\n    |> Stream.each(...)\n    |> Stream.run()\n\n    # change cursor field and set starting cursor\n    MyUser\n    |> MyRepo.cursor_based_stream(cursor_field: :email, after_cursor: \"foo@bar.com\")\n    |> Stream.each(...)\n    |> Stream.run()\n\n    # with multiple fields\n    MyUser\n    |> MyRepo.cursor_based_stream(cursor_field: [:email, :date_of_birth], after_cursor: %{email: \"foo@bar.com\"})\n    |> Stream.each(...)\n    |> Stream.run()\n\n    # select custom fields, remember to add cursor_field to select\n    MyUser\n    |> select([u], map(u, [:my_id, ...])\n    |> select_merge([u], ...)\n    |> MyRepo.cursor_based_stream(cursor_field: :my_id)\n    |> Stream.each(...)\n    |> Stream.run()\n\n    # pass custom options to Ecto.Repo.all/2\n    MyUser\n    |> MyRepo.cursor_based_stream(timeout: 60_000, prefix: \"public\")\n    |> Stream.each(...)\n    |> Stream.run()","ref":"EctoCursorBasedStream.html#c:cursor_based_stream/2-examples"},{"type":"type","title":"EctoCursorBasedStream.cursor_based_stream_opts/0","doc":"","ref":"EctoCursorBasedStream.html#t:cursor_based_stream_opts/0"},{"type":"extras","title":"EctoCursorBasedStream","doc":"# EctoCursorBasedStream\n\n![Build Status](https://img.shields.io/github/actions/workflow/status/allegro/ecto-cursor-based-stream/test.yml) [![Hex.pm](https://img.shields.io/hexpm/v/ecto_cursor_based_stream.svg)](https://hex.pm/packages/ecto_cursor_based_stream) [![Documentation](https://img.shields.io/badge/documentation-gray)](https://hexdocs.pm/ecto_cursor_based_stream/)\n\nCursor-based streaming of Ecto records, that does not require database transaction.\n\nGives you a [`cursor_based_stream/2`](https://hexdocs.pm/ecto_cursor_based_stream/EctoCursorBasedStream.html#c:cursor_based_stream/2) function that mimics [`Ecto.Repo.stream/2`](https://hexdocs.pm/ecto/Ecto.Repo.html#c:stream/2) interface.\n\nAdvantages in comparison to the standard `Ecto.Repo.stream/2`:\n\n- streaming can be stopped and continued at any point (by passing option `after_cursor: ...`),\n- works with tables that have milions of records.\n\nOnly limitation is that you have to supply a _cursor column or columns_ (by passing option `cursor_field: ...`, defaults to `:id`). Such a column(s):\n\n- must have unique values,\n- should have a database index. (So that sorting by it, and returning a number of rows larger than `x` is a performant operation.)","ref":"readme.html"},{"type":"extras","title":"Usage - EctoCursorBasedStream","doc":"1. Add `ecto_cursor_based_stream` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ecto_cursor_based_stream, \"~> 1.2.0\"}\n  ]\nend\n```\n\n2. Add `use EctoCursorBasedStream` to the module that uses `Ecto.Repo`:\n\n```elixir\ndefmodule MyRepo do\n  use Ecto.Repo\n  use EctoCursorBasedStream\nend\n```\n\n3. Stream the rows using `cursor_based_stream/2`:\n\n```elixir\nPost\n|> MyRepo.cursor_based_stream()\n|> Stream.each(...)\n|> Stream.run()\n```","ref":"readme.html#usage"},{"type":"extras","title":"Useful links - EctoCursorBasedStream","doc":"- [Documentation](https://hexdocs.pm/ecto_cursor_based_stream/EctoCursorBasedStream.html)\n- [Examples](https://github.com/allegro/ecto-cursor-based-stream/blob/main/test/ecto_cursor_based_stream_test.exs)","ref":"readme.html#useful-links"},{"type":"extras","title":"Contributing - EctoCursorBasedStream","doc":"","ref":"readme.html#contributing"},{"type":"extras","title":"Running tests - EctoCursorBasedStream","doc":"Run the following after cloning the repo:\n\n```sh\nmix deps.get\ndocker-compose up -d\nmix test\n```","ref":"readme.html#running-tests"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}